@@ -12,6 +12,7 @@ from .header import LasHeader
 from .laswriter import LasWriter
 from .point import record, dims, ExtraBytesParams, PointFormat
 from .point.dims import ScaledArrayView, SubFieldView, OLD_LASPY_NAMES
+from .point.record import DimensionNameValidity
 from .vlrs.vlrlist import VLRList
 
 logger = logging.getLogger(__name__)
@@ -303,6 +304,46 @@ class LasData:
                 New scales to be used. If not provided, the scales won't change.
         offsets: optional
                  New offsets to be used. If not provided, the offsets won't change.
+
+        Example
+        -------
+
+        >>> import laspy
+        >>> header = laspy.LasHeader()
+        >>> header.scales = np.array([0.1, 0.1, 0.1])
+        >>> header.offsets = np.array([0, 0 ,0])
+
+        >>> las = laspy.LasData(header=header)
+        >>> las.x = [10.0]
+        >>> las.y = [20.0]
+        >>> las.z = [30.0]
+
+        >>> # X = (x - x_offset) / x_scale
+        >>> assert np.all(las.xyz == [[10.0, 20., 30]])
+        >>> assert np.all(las.X == [100])
+        >>> assert np.all(las.Y == [200])
+        >>> assert np.all(las.Z == [300])
+
+        We change the scales (only changing x_scale here)
+        but not the offsets.
+
+        The xyz coordinates (double) are the same (minus possible rounding with actual coordinates)
+        However the integer coordinates changed
+
+        >>> las.change_scaling(scales=[0.01, 0.1, 0.1])
+        >>> assert np.all(las.xyz == [[10.0, 20., 30]])
+        >>> assert np.all(las.X == [1000])
+        >>> assert np.all(las.Y == [200])
+        >>> assert np.all(las.Z == [300])
+
+        Same idea if we change the offsets, the xyz do not change
+        but XYZ does
+
+        >>> las.change_scaling(offsets=[0, 10, 15])
+        >>> assert np.all(las.xyz == [[10.0, 20., 30]])
+        >>> assert np.all(las.X == [1000])
+        >>> assert np.all(las.Y == [100])
+        >>> assert np.all(las.Z == [150])
         """
         self.points.change_scaling(scales, offsets)
 
@@ -344,25 +385,19 @@ class LasData:
         eg: user tries to set the red field of a file with point format 0:
         an error is raised
         """
-
-        try:
-            key = OLD_LASPY_NAMES[key]
-        except KeyError:
-            pass
-
-        if (
-            key in self.point_format.dimension_names
-            or key in self.points.array.dtype.names
-        ):
-            self.points[key] = value
-        elif key in ("x", "y", "z"):
+        if key in ("x", "y", "z"):
             # It is possible that user created a `LasData` object
             # via `laspy.create`, and changed the headers offsets and scales
             # values afterwards. So we need to sync the points's record.
             self.points.offsets = self.header.offsets
             self.points.scales = self.header.scales
             self.points[key] = value
-        elif key in dims.DIMENSIONS_TO_TYPE:
+            return
+
+        name_validity = self.points.validate_dimension_name(key)
+        if name_validity == DimensionNameValidity.Valid:
+            self[key] = value
+        elif name_validity == DimensionNameValidity.Unsupported:
             raise ValueError(
                 f"Point format {self.point_format} does not support {key} dimension"
             )
